@@ -2,6 +2,7 @@
 import os
 import shutil
 from unittest import TestCase
+import pytest
 from pytest import raises
 from atropos.commands import execute_cli, get_command
 from tests.utils import (
@@ -606,3 +607,13 @@ class IssueTests(TestCase):
             expected2='issue68.2.fq',
             aligners=['insert'],
         )
+
+    @pytest.mark.timeout(10)
+    def test_issue122(self):
+        run_paired(
+            "--threads 2 --preserve-order --no-default-adapters -a TTAGACATAT -A CAGTGGAGTA",
+            in1="empty.fastq",
+            in2="empty.fastq",
+            expected1="empty.fastq",
+            expected2="empty.fastq",
+        )
\ No newline at end of file
